﻿/******************************************************************************
 * 文件名称: HisMainBusiness.cs
 * 文件说明: HIS主体交易的封装
 * 当前版本: V1.0
 * 创建日期: 2022-06-20
 *
 * * 2020-06-20: 增加 HisMainBusiness 类
 * *****  2020-06-20: 增加 CardReader 方法，获取URL地址,USER信息，地方版SSCard.dll使用
 * *****  2020-06-20: 增加 CardReader 方法重载，国家版电子凭证使用
 * *****  2020-06-20: 增加 Init 方法，SSCard环境初始化
 * *****  2020-06-20: 增加 ReadCardBas 方法，读社保卡
 * *****  2020-06-20: 增加 VerifyPIN 方法，验证密码
 * *****  2020-06-20: 增加 ChangePIN 方法，修改密码
 * *****  2020-06-20: 增加 ReadSFZ 方法，读身份证
 * *****  2020-06-20: 增加 GetQRBase 方法，读二维码
 * *****  2020-06-20: 增加 NationEcTrans 方法，读电子凭证（国家版）


******************************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Newtonsoft.Json.Linq;
using PTMedicalInsurance.Common;
using PTMedicalInsurance.Helper;
using PTMedicalInsurance.Variables;
using PTMedicalInsurance.Forms;
using System.Data;
using System.Reflection;
using System.IO;
using System.IO.Compression;

namespace PTMedicalInsurance.Business
{
    class HisMainBusiness
    {
        private InvokeHelper invoker = new InvokeHelper();
        private HisIrisServices hIS = new HisIrisServices();
        private MIIrisServices mIS = new MIIrisServices();
        private CenterBusiness cBus = new CenterBusiness();
        /// <summary>
        /// 获取HIS的IRIS配置信息
        /// </summary>
        /// <param name="con"></param>
        public void getHisConfingue(out IrisConfigue con)
        {
            IniFile ini = new IniFile(Global.curEvt.path + @"\INSUConfigure.ini");
            con.ip = ini.ReadValue("HIS", "ip");
            con.url = ini.ReadValue("HIS", "url");
            con.authorization = ini.ReadValue("HIS", "authorization");
        }

        /// <summary>
        /// 获取医保平台的iris配置信息
        /// </summary>
        /// <param name="con"></param>
        public void getInsuConfingue(out IrisConfigue con)
        {
            IniFile ini = new IniFile(Global.curEvt.path + @"\INSUConfigure.ini");
            con.ip = ini.ReadValue("INSU", "ip");
            con.url = ini.ReadValue("INSU", "url");
            con.authorization = ini.ReadValue("INSU", "authorization");
        }

        public void getLocalConfingue()
        {
            IniFile ini = new IniFile(Global.curEvt.path + @"\INSUConfigure.ini");
            Global.curEvt.ext.isOpenAnalysis = (ini.ReadValue("PreAndInProcessAnalysis", "isOpenAnalysis") == "1");
            Global.curEvt.enabledDebug = ("1".Equals(ini.ReadValue("YB", "debug")));
            Global.curEvt.testMode = ("1".Equals(ini.ReadValue("YB", "test")));
        }

        /// <summary>
        /// 获取Session信息（目前只获取操作员信息）
        /// </summary>
        /// <param name="joSession"></param>
        public void getSessionInfo(JObject joSession)
        {
            Global.user.ID = JsonHelper.getDestValue(joSession, "userCode");
            Global.user.name = JsonHelper.getDestValue(joSession, "userName");
            Global.user.type = "3";

            Global.user.officeID = JsonHelper.getDestValue(joSession, "locID");
            Global.user.officeName = JsonHelper.getDestValue(joSession, "locDesc");
        }

        /// <summary>
        /// 根据壳程序传入的入参设置全局接口变量
        /// </summary>
        /// <param name="joInpar"></param>
        /// <param name="inf"></param>
        public void setGlobalInterface(JObject joInpar,out  InterfaceInfo inf)
        {
            inf = new InterfaceInfo();
            inf.hospitalDr = int.Parse(JsonHelper.getDestValue(joInpar, "HospitalDr"));
            inf.hisHospitalDr = int.Parse(JsonHelper.getDestValue(joInpar, "hisHospitalDr"));
            inf.hospitalName = JsonHelper.getDestValue(joInpar, "HospitalName");
            inf.interfaceDr = int.Parse(JsonHelper.getDestValue(joInpar, "ID"));
            inf.interfaceID = JsonHelper.getDestValue(joInpar, "InterfaceID");
            //inf.oper = "3";
            inf.centerURL = JsonHelper.getDestValue(joInpar, "CenterURL");
            inf.ecURL = JsonHelper.getDestValue(joInpar, "CenterURL2");
            inf.areaCode = JsonHelper.getDestValue(joInpar, "AreaCode");
            inf.dllName = JsonHelper.getDestValue(joInpar, "DLLName");
            inf.hospitalNO = JsonHelper.getDestValue(joInpar, "HospitalNO");
            inf.version = "v1.0";
            inf.signatureType = "SM3";
            inf.recivedSystem = "YBXT";

            ////测试环境
            //inf.privateKey = "2af5cca6-de33-461d-a086-6423b92abbd6";
            //inf.Secret = "d82e88f67505b976a5781b725e9eed6da9b43601f21e7daae730a0d72c724126";
            //inf.centerURL = "http://10.126.40.47/tst/hsa/hgs/";
            //inf.cardURL = "http://10.126.105.66/hsa-hgs-adapt/api/card/initDll";   //测试环境

            //正式环境
            inf.privateKey = "0d2caf81-3c1b-4a7f-8955-d807eee0720e";
            inf.Secret = "dc80fc02f4feac82bb0c4a6e778292c0c33e8e0389a41f5c3865963b569841ec";
            inf.cardURL = "http://10.126.34.69/hsa-hgs-adapt/api/card/initDll";

            inf.dev_safe_info = "";
            inf.dev_no = Global.curEvt.mac;
            inf.cainfo = "";

            //查询区域通用目录 
            JObject joRtn = mIS.getInterface_TY(Global.inf.interfaceDr.ToString(), "");
            inf.interfaceDr_TY = int.Parse(JsonHelper.getDestValue(joRtn, "InsuCurrencyCataLogue"));    //通用目录编码 6              
            if (inf.interfaceDr_TY == 0)
            {
                inf.interfaceDr_TY = inf.interfaceDr;
            }          
        }

        /// <summary>
        /// 根据壳程序传入的入参设置全局接口变量
        /// </summary>
        /// <param name="joInpar"></param>
        /// <param name="inf"></param>
        public void setGlobalInterface_Plat(JObject joInpar, out InterfaceInfo inf)
        {
            inf = new InterfaceInfo();            

            inf.hospitalDr = int.Parse(JsonHelper.getDestValue(joInpar, "hospitalID"));            
            inf.hisHospitalDr = int.Parse(JsonHelper.getDestValue(joInpar, "hisHospitalDr"));            
            inf.hospitalName = JsonHelper.getDestValue(joInpar, "hospitalName");
            inf.interfaceDr = int.Parse(JsonHelper.getDestValue(joInpar, "medInsuInterfaceID"));            
            inf.interfaceID = JsonHelper.getDestValue(joInpar, "interfaceNO");
            //inf.oper = "3";
            inf.centerURL = JsonHelper.getDestValue(joInpar, "centerURL");
            inf.ecURL = JsonHelper.getDestValue(joInpar, "centerURL2");
            inf.areaCode = JsonHelper.getDestValue(joInpar, "areaCode");
            inf.dllName = JsonHelper.getDestValue(joInpar, "dLLName");
            inf.hospitalNO = JsonHelper.getDestValue(joInpar, "hospitalNO");
            inf.version = "v1.0";
            inf.signatureType = "SM3";
            inf.recivedSystem = "YBXT";

            ////测试环境
            //inf.privateKey = "2af5cca6-de33-461d-a086-6423b92abbd6";
            //inf.Secret = "d82e88f67505b976a5781b725e9eed6da9b43601f21e7daae730a0d72c724126";
            //inf.centerURL = "http://10.126.40.47/tst/hsa/hgs/";
            //inf.cardURL = "http://10.126.105.66/hsa-hgs-adapt/api/card/initDll";   //测试环境

            //正式环境
            inf.privateKey = "0d2caf81-3c1b-4a7f-8955-d807eee0720e";
            inf.Secret = "dc80fc02f4feac82bb0c4a6e778292c0c33e8e0389a41f5c3865963b569841ec";
            inf.cardURL = "http://10.126.34.69/hsa-hgs-adapt/api/card/initDll";

            inf.dev_safe_info = "";
            inf.dev_no = Global.curEvt.mac;
            inf.cainfo = "";

            //查询区域通用目录 
            JObject joRtn = mIS.getInterface_TY(Global.inf.interfaceDr.ToString(), "");
            if (int.TryParse(JsonHelper.getDestValue(joRtn, "InsuCurrencyCataLogue"), out inf.interfaceDr_TY) == false)
            {
                inf.interfaceDr_TY = inf.interfaceDr;
            }
        }

        /// <summary>
        /// 初始化环境
        /// </summary>
        /// <param name="inParam"></param>
        /// <returns></returns>
        public  int initEnvironment(string inParam,out string error)
        {
            error = string.Empty;
            try
            {              
                //获取HIS和医保平台的IRIS配置信息
                getHisConfingue(out Global.hisConfig);
                
                getInsuConfingue(out Global.insuConfig);

                getLocalConfingue();

                JObject joInparam = JObject.Parse(inParam);

                JObject JoSession = JObject.Parse(JsonHelper.getDestValue(joInparam, "session[0]"));
                string  hisHospitalDr = JsonHelper.getDestValue(JoSession, "hospID");

                JObject joInterface = JObject.Parse(JsonHelper.getDestValue(joInparam, "interfaceinfo"));

                joInterface.Add("hisHospitalDr", hisHospitalDr); 

                if (joInterface != null)
                {
                    //获取当前接口详细信息
                    if (Global.businessType == "P1")
                    {                        
                        setGlobalInterface_Plat(joInterface, out Global.inf);  
                    }
                    else
                    {                        
                        setGlobalInterface(joInterface, out Global.inf);
                    }
                    
                    JObject joSession = JObject.Parse(JsonHelper.getDestValue(joInparam, "session[0]"));
                    if (joSession != null)
                    {
                        //获取当前用户信息                        
                        getSessionInfo(joSession);
                        return 0;
                    }
                    else
                    {
                        error = "初始化环境失败：session获取失败！";
                        return -1;
                    }

                }
                else
                {
                    error = "初始化环境失败：interfaceinfo获取失败！";
                    return -1;
                }
                
            }
            catch (Exception ex)
            {
                error ="初始化环境失败：" + ex.Message;
                return -1;
            }
            
        }

        /// <summary>
        /// 读卡
        /// </summary>
        /// <returns></returns>
        public int readCard(out string outParam)
        {
            outParam = "";
            string errorMsg = "";
            string sInput = "";
            string sOutPar;
            Global.pat.bChangePIN = false;
            JObject joCardInfo = new JObject();
            ChooseCard cc = new ChooseCard();
            CardReader_HN CardRead = new CardReader_HN();
            //g读卡
            CardReader_GS CardRead_GS = new CardReader_GS(Global.inf.cardURL,Global.inf.areaCode);
            InvokeHelper invoker = new InvokeHelper();
            try
            {
                if (cc.ShowDialog() == DialogResult.OK)
                {
                    #region【电子凭证】
                    if (cc.sP_CertType == "01")
                    {

                        Global.pat.mdtrtcertType = "01";
                        Global.pat.certType = "01";
                        Random rd = new Random();
                        int iNum = rd.Next();
                        string sLSH = DateTime.Now.ToString("YYYYMMDDHHNNSS") + Global.inf.hospitalNO + iNum.ToString().Substring(0, 5);

                        joCardInfo.Add("operatorId", Global.user.ID);
                        joCardInfo.Add("operatorName", Global.user.name);
                        joCardInfo.Add("officeId", Global.user.officeID);
                        joCardInfo.Add("officeName", Global.user.officeName);
                        joCardInfo.Add("orgId", Global.inf.hospitalNO);
                        joCardInfo.Add("businessType", cc.sL_CertCodeType);

                        //认证方式
                        //二维码
                        if (cc.EcCertDecodeType == "0")
                        {
                            joCardInfo.Add("deviceType", cc.sL_CertCodeType);

                            //调用电子凭证动态库NationECCode.dll获取身份信息                        
                            JObject JoInput = new JObject();
                            JoInput.Add("orgId", Global.inf.hospitalNO);
                            JoInput.Add("transType", cc.sL_EcCertDecodeTradeNO);
                            JoInput.Add("data", joCardInfo);
                            JoInput.Add("extra", "");

                            //MessageBox.Show("电子凭证解码地址：" + Global.inf.ecURL + ",入参：" + JoInput.ToString());

                            if (Global.pat.bPresCirul == true)
                            {
                                //测试环境地址
                                //Global.inf.ecURL = "http://10.126.40.13:8080/localcfc/api/hsecfc/localQrCodeQuery";
                            }

                            CardRead_GS.NationEcTrans(Global.inf.ecURL, JoInput.ToString(), out sOutPar);
                            if (JsonHelper.getDestValue(JObject.Parse(sOutPar), "code") != "0")
                            {
                                outParam = "调用电子凭证失败：" + sOutPar;
                                return -1;
                            }

                            Global.pat.ecToken = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.ecToken");
                            Global.pat.name = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.userName");
                            Global.pat.IDType = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.idType");
                            Global.pat.IDNO = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.idNo");
                            Global.pat.insuplc_admdvs = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.insuOrg");

                            Global.pat.mdtrtcertNO = Global.pat.ecToken;
                            Global.pat.certType = Global.pat.IDType;
                            Global.pat.certNO = Global.pat.IDNO;
                        }
                        //电子凭证
                        else if (cc.EcCertDecodeType == "1")
                        {
                            joCardInfo.Add("outBizNo", sLSH);
                            joCardInfo.Add("extData", "");
                        }
                        //刷脸、先授权后解码
                        else if (cc.EcCertDecodeType == "2") //刷脸 
                        {
                            //20241212 老的 先注释掉 用郑州的代码先
                            //JObject JoGetAuthInfo = new JObject();
                            //JoGetAuthInfo.Add("outBizNo", sLSH);
                            //JoGetAuthInfo.Add("extData", "");

                            //1.调用人脸识别认证 cn.nhsa.ec.auth
                            joCardInfo.Add("outBizNo", sLSH);
                            joCardInfo.Add("extData", "");

                            JObject JoInputAuth = new JObject();
                            JoInputAuth.Add("orgId", Global.inf.hospitalNO);
                            JoInputAuth.Add("transType", "cn.nhsa.ec.auth");
                            JoInputAuth.Add("data", joCardInfo);
                            JoInputAuth.Add("extra", "");

                            //调用刷脸认证接口cn.nhsa.ec.auth返回授权信息authNo
                            CardRead.NationEcTrans(Global.inf.ecURL, JoInputAuth.ToString(), out sOutPar);
                            if (JsonHelper.getDestValue(JObject.Parse(sOutPar), "code") != "0")
                            {
                                outParam = "调用刷脸认证接口cn.nhsa.ec.auth失败：" + sOutPar;
                                return -1;
                            }
                            else
                            {
                                //成功返回授权信息
                                /*20241212 老的 先注释掉 用郑州的代码先
                                string auth = sOutPar;
                                JObject JoAuthOut = new JObject();
                                JoAuthOut.Add("outBizNo", DateTime.Now.ToString("YYYYMMDDHHNNSS") + Global.inf.hospitalNO + iNum.ToString().Substring(0, 5));
                                JoAuthOut.Add("authNo", auth);
                                JoAuthOut.Add("extData", "");

                                joCardInfo = (JObject)JoAuthOut.DeepClone();
                                */

                                //成功返回授权信息
                                string auth = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.authNo");
                                JObject JoAuthCheckInput = new JObject();
                                JoAuthCheckInput.Add("orgId", Global.inf.hospitalNO);
                                JoAuthCheckInput.Add("outBizNo", DateTime.Now.ToString("yyyyMMddHHmmss") + Global.inf.hospitalNO + iNum.ToString().Substring(0, 5));
                                JoAuthCheckInput.Add("authNo", auth);
                                JoAuthCheckInput.Add("businessType", cc.sL_CertCodeType);
                                JoAuthCheckInput.Add("operatorId", Global.user.ID);
                                JoAuthCheckInput.Add("operatorName", Global.user.name);
                                JoAuthCheckInput.Add("officeId", Global.user.officeID);
                                JoAuthCheckInput.Add("officeName", Global.user.officeName);
                                JoAuthCheckInput.Add("extData", "");

                                JObject JoAuthChck = new JObject();
                                JoAuthChck.Add("orgId", Global.inf.hospitalNO);
                                JoAuthChck.Add("transType", "cn.nhsa.auth.check");
                                JoAuthChck.Add("data", JoAuthCheckInput);
                                JoAuthChck.Add("extra", "");

                                //2.调用刷脸认证接口cn.nhsa.ec.auth返回授权信息authNo
                                CardRead.NationEcTrans(Global.inf.ecURL, JoAuthChck.ToString(), out sOutPar);
                                if (JsonHelper.getDestValue(JObject.Parse(sOutPar), "code") != "0")
                                {
                                    outParam = "调用刷脸授权获取医保身份接口cn.nhsa.auth.check失败：" + sOutPar;
                                    return -1;
                                }
                                else
                                {
                                    Global.pat.ecToken = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.ecToken");
                                    Global.pat.name = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.userName");
                                    Global.pat.IDType = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.idType");
                                    Global.pat.IDNO = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.idNo");
                                    Global.pat.insuplc_admdvs = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.insuOrg");
                                    Global.pat.mdtrtcertNO = Global.pat.ecToken;
                                    Global.pat.certType = Global.pat.IDType;
                                    Global.pat.certNO = Global.pat.IDNO;
                                }

                            }


                        }
                        else
                        {

                            //调用电子凭证动态库NationECCode.dll获取身份信息                        
                            JObject JoInput = new JObject();
                            JoInput.Add("orgId", Global.inf.hospitalNO);
                            JoInput.Add("transType", cc.sL_EcCertDecodeTradeNO);
                            JoInput.Add("data", joCardInfo);
                            JoInput.Add("extra", "");

                            //MessageBox.Show("电子凭证解码地址：" + Global.inf.ecURL + ",入参：" + JoInput.ToString());

                            if (Global.pat.bPresCirul == true)
                            {
                                //测试环境地址
                                //Global.inf.ecURL = "http://10.126.40.13:8080/localcfc/api/hsecfc/localQrCodeQuery";
                            }

                            CardRead_GS.NationEcTrans(Global.inf.ecURL, JoInput.ToString(), out sOutPar);
                            if (JsonHelper.getDestValue(JObject.Parse(sOutPar), "code") != "0")
                            {
                                outParam = "调用电子凭证失败：" + sOutPar;
                                return -1;
                            }

                            Global.pat.ecToken = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.ecToken");
                            Global.pat.name = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.userName");
                            Global.pat.IDType = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.idType");
                            Global.pat.IDNO = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.idNo");
                            Global.pat.insuplc_admdvs = JsonHelper.getDestValue(JObject.Parse(sOutPar), "data.insuOrg");

                            Global.pat.mdtrtcertNO = Global.pat.ecToken;
                            Global.pat.certType = Global.pat.IDType;
                            Global.pat.certNO = Global.pat.IDNO;
                        }

                    }
                    #endregion

                    #region【身份证】
                    if (cc.sP_CertType == "02")
                    {
                        if (Global.pat.card.SearchAdmKey != "")
                            Global.pat.insuplc_admdvs = Global.pat.card.SearchAdmCode;

                        Global.pat.mdtrtcertType = "02";
                        Global.pat.mdtrtcertNO = cc.sP_CertNO;
                        Global.pat.certType = "01";
                        Global.pat.certNO = cc.sP_IDNO;
                        Global.pat.name = cc.PatName;
                        Global.pat.card.SN = "";
                    }
                    #endregion

                    #region【社保卡】
                    if (cc.sP_CertType == "03")
                    {
                        //初始化读卡函数
                        if (CardRead_GS.Init() != 0)
                        {
                            outParam = "读卡初始化Init失败！";
                            return -1;
                        }

                        string basInfo = "", regInfo = "";
                        int a = CardRead_GS.ReadCardBas(out basInfo, out regInfo);

                        if (a != 0)
                        {
                            outParam = "社保卡读卡异常ReadCardBas(本地)：" + basInfo;
                            return -1;
                        }
                        else
                        {
                            //【社保卡读卡成功返回】
                            Global.pat.insuplc_admdvs = tools.getDestPosStrBySpliter(basInfo, 1);    //参保地
                            Global.pat.card.NO = tools.getDestPosStrBySpliter(basInfo, 2);           //社会保障号码
                            Global.pat.certNO = tools.getDestPosStrBySpliter(basInfo, 3);            //卡号
                            Global.pat.card.SN = tools.getDestPosStrBySpliter(basInfo, 4);           //卡识别码                                                
                            Global.pat.name = tools.getDestPosStrBySpliter(basInfo, 5);              //姓名
                            Global.pat.certType = "01";
                            Global.pat.mdtrtcertType = "03";
                            Global.pat.mdtrtcertNO = Global.pat.certNO;
                            Global.pat.certNO = Global.pat.card.NO;
                        }
                    }
                    #endregion

                    //是否异地
                    Global.pat.bOtherProvLocal = cc.bOtherProvLocal;
                }
                else
                {
                    outParam = "收款员取消读卡！";
                    return -1;
                }
               
                #region【1101获取身份信息】
                JObject joData = new JObject();
                joData.Add("mdtrt_cert_type", Global.pat.mdtrtcertType);
                joData.Add("mdtrt_cert_no", Global.pat.mdtrtcertNO);
                joData.Add("card_sn", Global.pat.card.SN);
                joData.Add("begntime", "");
                joData.Add("psn_cert_type", Global.pat.certType);
                joData.Add("certno", Global.pat.certNO); //证件号码
                joData.Add("psn_name", Global.pat.name);
                JObject joInput = new JObject();
                joInput.Add("data",joData);

                //InvokeHelper invoker = new InvokeHelper();
                JObject joRtn = invoker.invokeCenterService("1101", JsonHelper.setCenterInpar("1101", joInput));
                if (JsonHelper.parseCenterRtnValue(joRtn, out errorMsg) != 0)
                {
                    outParam = "读卡失败：" + errorMsg;                    
                    return -1;
                }
                else
                {
                    //joRtn.Add("cardinfo",joCardInfo);                    
                    outParam = joRtn.ToString();                   
                    Global.pat.insuplc_admdvs = JsonHelper.getDestValue(JObject.Parse(outParam), "output.insuinfo.insuplc_admdvs"); //参保地
                    Global.pat.certNO = JsonHelper.getDestValue(JObject.Parse(outParam), "output.baseinfo.certno"); //证件号码
                    Global.pat.psn_no = JsonHelper.getDestValue(JObject.Parse(outParam), "output.baseinfo.psn_no"); //人员编号
                    outParam = outParam.Replace(@"\", "").Replace("\"{", "{").Replace("}\"", "}");                                     
                    return 0;
                }
                #endregion
            }
            catch (Exception ex)
            {
                outParam = "异常：" + ex.Message;                
                return -1;
            }
        }

        /// <summary>
        /// 展示患者信息界面，转换相关编码为中文，并获取操作员选中的信息
        /// </summary>
        /// <param name="patInfo"></param>
        /// <param name="outparam"></param>
        /// <returns></returns>
        public int showPatInfo(String patInfo,out string outparam )
        {
            outparam = "";
            //展示信息界面
            JObject joOutparam = JObject.Parse(patInfo);
            JObject joOutput = JObject.Parse(JsonHelper.getDestValue(joOutparam, "output"));
            //编码转换
            JArray jaConvertCode = new JArray();
            JObject joConvertCodeParam = new JObject();
            joConvertCodeParam.Add("hospitalDr", Global.inf.hospitalDr);
            joConvertCodeParam.Add("interfaceDr", Global.inf.interfaceDr);
            joConvertCodeParam.Add("output", joOutput);
            jaConvertCode.Add(joConvertCodeParam);
            JObject joConvertCode = new JObject();
            joConvertCode.Add("code", "09010054");
            joConvertCode.Add("params", jaConvertCode);
            InvokeHelper invoker = new InvokeHelper();
            JObject joConvertCodeRtn = invoker.invokeInsuService(joConvertCode.ToString(), "患者信息编码转换");

            PatientInsuInfo frmPatientInfo = new PatientInsuInfo(joConvertCodeRtn);
            //相关信息转换到登记面板
            try
            {
                if (frmPatientInfo.ShowDialog() == DialogResult.OK)
                {
                    //原来用的是转换后的
                    //frmPatientInfo.joSelectedInsuInfo.Add("mdtrt_cert_type", Global.pat.mdtrtcertType);
                    //frmPatientInfo.joSelectedInsuInfo.Add("mdtrt_cert_no", Global.pat.mdtrtcertNO);
                    //joOutparam.Property("output").AddBeforeSelf(new JProperty("selectedInsuInfo", frmPatientInfo.joSelectedInsuInfo));

                    //现在用转换之前的
                    JArray jaInsuInfo = JArray.Parse(JsonHelper.getDestValue(joOutput, "insuinfo"));
                    JArray jaIdInfo = JArray.Parse(JsonHelper.getDestValue(joOutput, "idetinfo"));
                    JObject joSelectedInsuInfo = (JObject)jaInsuInfo[frmPatientInfo.insuInfoIndex];
                    JObject joSelectedIdInfo;
                    if (jaIdInfo.Count != 0)
                    {
                        joSelectedIdInfo = (JObject)jaIdInfo[frmPatientInfo.idInfoIndex];
                    }
                    else
                    {
                        joSelectedIdInfo = new JObject();
                    }
                    joSelectedInsuInfo.Add("mdtrt_cert_type", Global.pat.mdtrtcertType);
                    joSelectedInsuInfo.Add("mdtrt_cert_no", Global.pat.mdtrtcertNO);
                    joOutparam.Property("output").AddBeforeSelf(new JProperty("selectedInsuInfo", joSelectedInsuInfo));
                    joOutparam.Property("output").AddBeforeSelf(new JProperty("selectedIdInfo", joSelectedInsuInfo));

                    Global.pat.balc = JsonHelper.getDestValue(joSelectedInsuInfo, "balc");
                    Global.pat.insuType = JsonHelper.getDestValue(joSelectedInsuInfo, "insutype");
                    Global.pat.insuplc_admdvs = JsonHelper.getDestValue(joSelectedInsuInfo, "insuplc_admdvs");

                    outparam = joOutparam.ToString();

                    return 0;
                }
                else
                {
                    outparam = JsonHelper.setExceptionJson(-1, "门诊读卡", "收款员取消读卡").ToString();
                    return -1;
                }
            }
            catch (Exception e)
            {
                outparam = JsonHelper.setExceptionJson(-1, "患者参保信息展示", e.Message).ToString();
                return -1;
            }
        }

        /// <summary>
        /// 患者读卡并获取中心信息（1101）后设置PAT结构体
        /// </summary>
        /// <param name="patInfo"></param>
        /// <param name="errMsg"></param>
        /// <returns></returns>
        public int setGlobalPatAfaterShowPatInfo(string patInfo,out string errMsg)
        {
            JObject joPatInfo;
            errMsg = "";
            try
            {                
                joPatInfo = JObject.Parse(patInfo);
                
                //基本信息
                Global.pat.certNO = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.certno");
                Global.pat.psn_no = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.psn_no");
                Global.pat.gend = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.gend");
                Global.pat.brdy = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.brdy");
                Global.pat.naty = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.naty");
                Global.pat.age = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.age");
                Global.pat.certType = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.psn_cert_type");

                //if (Global.pat.name != JsonHelper.getDestValue(joPatInfo, "output.baseinfo.psn_name"))
                //{
                //    errMsg = "患者姓名(" + Global.pat.name + ")与参保人姓名(" + JsonHelper.getDestValue(joPatInfo, "output.baseinfo.psn_name") + ")不一致，请检查！";
                //    return -1;
                //}

                Global.pat.name = JsonHelper.getDestValue(joPatInfo, "output.baseinfo.psn_name");
                //选中的参保信息
                Global.pat.insuplc_admdvs = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.insuplc_admdvs");
                Global.pat.psn_insu_date = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.psn_insu_date");
                Global.pat.cvlserv_flag = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.cvlserv_flag");
                Global.pat.balc = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.balc");
                Global.pat.emp_name = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.emp_name");
                Global.pat.psn_type = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.psn_type");
                Global.pat.psn_insu_stas = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.psn_insu_stas");
                Global.pat.insuType = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.insutype");
                Global.pat.paus_insu_date = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.paus_insu_date");                

                //卡片信息
                //Global.pat.cardSN = JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo.paus_insu_date");
                //选中的身份信息
                return 0;
            }
            catch (Exception ex)
            {
                errMsg = "获取患者信息发生异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 显示住院患者登记面板，返回登记入参
        /// </summary>
        /// <param name="regInfo"></param>
        /// <returns></returns>
        public int showInPatRegisterForm(out string regInfo)
        {

            //调用服务获取门诊诊断信息
            string diagnoses;
            if (hIS.getPatDiagnoses(Global.pat, out diagnoses) != 0)
            {
                regInfo = JsonHelper.setExceptionJson(-1, "getPatDiagnoses", diagnoses).ToString();
                return -1;
            }
            JObject joHisRtnInfo = JObject.Parse(diagnoses);

            //组织登记入参
            JObject joReg = new JObject();
            try
            {
                JArray jaDiagnoses = JArray.Parse(JsonHelper.getDestValue(joHisRtnInfo, "diseinfo"));
                string dscg_maindiag_code  = "",dscg_maindiag_name = "";
                //  对诊断数组进行转换 转换下主要诊断
                JArray jaConvertedDiagnoses = new JArray();
                JArray jaInPatDiagnoses = new JArray();
                JArray jaOutPatDiagnoses = new JArray();
                for (int i = 0; i < jaDiagnoses.Count; i++)
                {
                  
                    string diag_type = JsonHelper.getDestValue((JObject)jaDiagnoses[i], "diag_type");
                    switch (diag_type)
                    {
                        case "1":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "西医主要诊断";
                                break;
                            }
                        case "2":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "西医其他诊断";
                                break;
                            }
                        case "3":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "中医主病诊断";
                                break;
                            }
                        case "4":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "中医主证诊断";
                                break;
                            }
                    }
                    

                    //if (jaDiagnoses[i]["diaTypeCode"].ToString() == "ADD") //ADD 入院 OPD 门诊
                    //{
                    //    jaConvertedDiagnoses.Add(jaDiagnoses[i]);
                    //}


                    if (jaDiagnoses[i]["diaTypeCode"].ToString() == "ADD") //ADD 入院 OPD 门诊
                    {
                        jaInPatDiagnoses.Add(jaDiagnoses[i]);
                    }
                    if (jaDiagnoses[i]["diaTypeCode"].ToString() == "OPD") //ADD 入院 OPD 门诊
                    {
                        jaOutPatDiagnoses.Add(jaDiagnoses[i]);
                    }
                }

                if (jaInPatDiagnoses.Count > 0)
                {
                    jaConvertedDiagnoses = (JArray)jaInPatDiagnoses.DeepClone();
                }
                else 
                {
                    jaConvertedDiagnoses = (JArray)jaOutPatDiagnoses.DeepClone();
                }

                for (int i = 0; i < jaConvertedDiagnoses.Count; i++)
                {
                    jaConvertedDiagnoses[i]["diag_srt_no"] = i + 1;
                    if (jaConvertedDiagnoses[i]["maindiag_flag"].ToString() == "1")
                    {
                        dscg_maindiag_code = jaConvertedDiagnoses[i]["diag_code"].ToString();
                        dscg_maindiag_name = jaConvertedDiagnoses[i]["diag_name"].ToString();
                    }
                }                

   
                //登记面板实例
                OutpatientRegistration frmReg = new OutpatientRegistration();
                frmReg.dblkcbxSettelmentWay.Enabled = false;

                DataTable dt = (DataTable)frmReg.dblkcbxInsuranceType.DataSource;
                int selectedIndex = 0;
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    if (dt.Rows[i]["Code"].ToString() == Global.pat.insuType)
                    {
                        selectedIndex = i;
                        frmReg.dblkcbxInsuranceType.Text = dt.Rows[i]["Name"].ToString();
                        frmReg.insuType = dt.Rows[i]["Code"].ToString();
                    }
                }

                DataTable dtMedType = (DataTable)frmReg.dblkcbxMedicalType.DataSource;
                for (int i = 0; i < dtMedType.Rows.Count; i++)
                {
                    if (dtMedType.Rows[i]["Code"].ToString() == "2101")
                    {
                        frmReg.dblkcbxMedicalType.Text = dtMedType.Rows[i]["Name"].ToString();
                        frmReg.med_type = dtMedType.Rows[i]["Code"].ToString();
                    }
                }
                frmReg.DtDiagnose = (DataTable)jaConvertedDiagnoses.ToObject(typeof(DataTable));
                //展示登记面板
                if (frmReg.ShowDialog() == DialogResult.OK)
                {
                    string diseCodg ="",diseName = "";

                    //兰州要求必须传病种。普通疾病传主诊断
                    if ((frmReg.med_type == "2101") && (frmReg.diseCodg == ""))                    
                    {
                        if (string.IsNullOrEmpty(frmReg.diseCodg))
                        {
                            diseCodg = JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_code");
                        };
                        if (string.IsNullOrEmpty(frmReg.diseName))
                        {
                            diseName = JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_name");
                        }
                    }

                    JObject joMdtrtinfo = new JObject();
                    joMdtrtinfo.Add("psn_no", Global.pat.psn_no);          //人员编号
                    joMdtrtinfo.Add("insutype", Global.pat.insuType);          //险种类型
                    joMdtrtinfo.Add("coner_name", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.coner_name"));          //联系人姓名
                    joMdtrtinfo.Add("tel", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.tel"));          //联系电话
                    joMdtrtinfo.Add("begntime", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.begntime"));          //开始时间
                    joMdtrtinfo.Add("mdtrt_cert_type", Global.pat.mdtrtcertType);          //就诊凭证类型
                    joMdtrtinfo.Add("mdtrt_cert_no", Global.pat.mdtrtcertNO);          //就诊凭证编号
                    joMdtrtinfo.Add("med_type", frmReg.med_type);          //医疗类别
                    joMdtrtinfo.Add("ipt_no", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.ipt_no"));          //住院号
                    joMdtrtinfo.Add("medrcdno", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.medrcdno"));          //病历号
                    joMdtrtinfo.Add("atddr_no", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.atddr_no"));          //主治医生编码
                    joMdtrtinfo.Add("chfpdr_name", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.chfpdr_name"));          //主诊医师姓名
                    joMdtrtinfo.Add("adm_diag_dscr", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_diag_dscr"));          //入院诊断描述
                    joMdtrtinfo.Add("adm_dept_codg", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_dept_codg"));          //入院科室编码
                    joMdtrtinfo.Add("adm_dept_name", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_dept_name"));          //入院科室名称
                    joMdtrtinfo.Add("adm_bed", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_bed"));          //入院床位
                    joMdtrtinfo.Add("dscg_maindiag_code", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_code"));          //住院主诊断代码
                    joMdtrtinfo.Add("dscg_maindiag_name", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_name"));          //住院主诊断名称
                    //joMdtrtinfo.Add("dscg_maindiag_code", dscg_maindiag_code);          //住院主诊断代码
                    //joMdtrtinfo.Add("dscg_maindiag_name", dscg_maindiag_name);          //住院主诊断名称
                    joMdtrtinfo.Add("main_cond_dscr", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.main_cond_dscr"));          //主要病情描述
                    joMdtrtinfo.Add("dise_codg", diseCodg);          //病种编码
                    joMdtrtinfo.Add("dise_name", diseName);          //病种名称
                    joMdtrtinfo.Add("oprn_oprt_code", frmReg.oprn_oprt_code);          //手术操作代码
                    joMdtrtinfo.Add("oprn_oprt_name", frmReg.oprn_oprt_name);          //手术操作名称
                    joMdtrtinfo.Add("fpsc_no", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.fpsc_no"));          //计划生育服务证号
                    joMdtrtinfo.Add("matn_type", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.matn_type"));          //生育类别
                    joMdtrtinfo.Add("birctrl_type", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.birctrl_type"));          //计划生育手术类别
                    joMdtrtinfo.Add("latechb_flag", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.latechb_flag"));          //晚育标志
                    joMdtrtinfo.Add("geso_val", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.geso_val"));          //孕周数
                    joMdtrtinfo.Add("fetts", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.fetts"));          //胎次
                    joMdtrtinfo.Add("fetus_cnt", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.fetus_cnt"));          //胎儿数
                    joMdtrtinfo.Add("pret_flag", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.pret_flag"));          //早产标志
                    joMdtrtinfo.Add("birctrl_matn_date", "");          //计划生育手术或生育日期 JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.birctrl_matn_date") 20230228医保中心校验传正确时间或空
                    joMdtrtinfo.Add("dise_type_code", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dise_type_code"));          //病种类型
                    joMdtrtinfo.Add("exp_content", "");          //字段扩展
                    joMdtrtinfo.Add("insuplc_admdvs", Global.pat.insuplc_admdvs);          //参保地医保区划
                    joMdtrtinfo.Add("mdtrtarea_admvs", Global.inf.areaCode);          //就医地医保区划

                    joMdtrtinfo.Add("patID", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.patID"));          //HIS患者ID

                    joReg.Add("mdtrtinfo", joMdtrtinfo);
                    joReg.Add("diseinfo", jaConvertedDiagnoses);

                    Global.pat.disCode = diseCodg;
                    Global.pat.disName = diseName;
                    Global.pat.medType = frmReg.med_type;

                    regInfo = joReg.ToString();
                    return 0;
                }
                else
                {
                    regInfo = "收费员取消操作！";
                    return -1;
                }
            }
            catch (Exception ex)
            {
                regInfo = "登记界面显示异常：" + ex.Message;
                return -1;
            }

        }

        /// <summary>
        /// 校验HIS信息与医保参保信息姓名是否一致
        /// </summary>
        /// <param name="insuName"></param>
        /// <param name="errMsg"></param>
        /// <returns></returns>
        public int checkName(string insuName, out string errMsg)
        {
            try
            {
                errMsg = "";
                string hisPatBaseInfo = "";

                if (hIS.GetHisPatBaseInfo(out hisPatBaseInfo) != 0)
                {
                    errMsg = hisPatBaseInfo;
                    return -1;
                }
                string hisPatName = JsonHelper.getDestValue(JObject.Parse(hisPatBaseInfo), "result.patName");
                if (hisPatName != insuName)
                {
                    errMsg = "患者姓名(" + hisPatName + ")与参保人姓名(" + insuName + ")不一致，请检查！";
                    return -1;
                }
                else
                {
                    return 0;
                }
            }
            catch (Exception ex)
            {
                errMsg = "核对HIS与医保平台姓名一致性发生异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 显示出院登记界面，返回出院登记和结算的入参
        /// </summary>
        /// <param name="joSumFee"></param>
        /// <param name="outParam"></param>
        /// <returns></returns>
        public int showDischargeRegisterForm(string funNO, JObject joSumFee, JObject joHisRtnInfo, out string outParam)
        {
            outParam = "";
            string errMsg;
            try
            {
                JArray jaDiagnoses = JArray.Parse(JsonHelper.getDestValue(joHisRtnInfo, "diseinfo"));
                //选择结算方式等
                OutpatientRegistration frmReg = new OutpatientRegistration();
                frmReg.dblkcbxInsuranceType.Enabled = true;

                DataTable dt = (DataTable)frmReg.dblkcbxInsuranceType.DataSource;
                int selectedIndex = 0;

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    if (dt.Rows[i]["Code"].ToString() == Global.pat.insuType)
                    {
                        selectedIndex = i;

                        frmReg.dblkcbxInsuranceType.Text = dt.Rows[i]["Name"].ToString();
                        frmReg.insuType = dt.Rows[i]["Code"].ToString();
                    }
                }

                dt = (DataTable)frmReg.dblkcbxMedicalType.DataSource;
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    if (dt.Rows[i]["Code"].ToString() == Global.pat.medType)
                    {
                        frmReg.dblkcbxMedicalType.Text = dt.Rows[i]["Name"].ToString();
                        frmReg.med_type = dt.Rows[i]["Code"].ToString();
                    }
                }

                if (Global.pat.RYorCY == "1")
                {
                    DataTable dtMedType = (DataTable)frmReg.dblkcbxMedicalType.DataSource;
                    for (int i = 0; i < dtMedType.Rows.Count; i++)
                    {
                        if (dtMedType.Rows[i]["Code"].ToString() == "2101")
                        {
                            frmReg.dblkcbxMedicalType.Text = dtMedType.Rows[i]["Name"].ToString();
                            frmReg.med_type = dtMedType.Rows[i]["Code"].ToString();
                        }
                    }
                }

                DataTable dtSettlWay = (DataTable)frmReg.dblkcbxSettelmentWay.DataSource;
                for (int i = 0; i < dtSettlWay.Rows.Count; i++)
                {
                    if (dtSettlWay.Rows[i]["Code"].ToString() == "01")
                    {
                        frmReg.dblkcbxSettelmentWay.Text = dtSettlWay.Rows[i]["Name"].ToString();
                        frmReg.psn_setlway = dtSettlWay.Rows[i]["Code"].ToString();
                    }
                }

                DataTable DTdiseCodg = (DataTable)frmReg.dblkcbxDisease.DataSource;
                for (int i = 0; i < DTdiseCodg.Rows.Count; i++)
                {
                    if (DTdiseCodg.Rows[i]["Code"].ToString() == Global.pat.DiseasecCode)
                    {
                        frmReg.dblkcbxDisease.Text = DTdiseCodg.Rows[i]["Name"].ToString();
                        frmReg.diseCodg = DTdiseCodg.Rows[i]["Code"].ToString();
                    }
                }

                if (frmReg.dblkcbxInsuranceType.Text != "")
                    frmReg.dblkcbxInsuranceType.Enabled = false;
                if (frmReg.dblkcbxMedicalType.Text != "")
                    frmReg.dblkcbxMedicalType.Enabled = false;
                if (frmReg.dblkcbxDisease.Text != "")
                    frmReg.dblkcbxDisease.Enabled = false;

                //  对诊断数组进行转换 转换下主要诊断
                JArray jaConvertedDiagnoses = new JArray();
                string dscg_maindiag_code = "", dscg_maindiag_name = "";
                for (int i = 0; i < jaDiagnoses.Count; i++)
                {
                    string diag_type = JsonHelper.getDestValue((JObject)jaDiagnoses[i], "diag_type");
                    switch (diag_type)
                    {
                        case "1":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "西医主要诊断";
                                break;
                            }
                        case "2":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "西医其他诊断";
                                break;
                            }
                        case "3":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "中医主病诊断";
                                break;
                            }
                        case "4":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "中医主证诊断";
                                break;
                            }
                    }

                    //出院登记 取出院诊断信息
                    if (Global.pat.RYorCY == "2")
                    {
                        if (jaDiagnoses[i]["diaTypeCode"].ToString() == "DIS") //ADD 入院 OPD 门诊 DIS 出院
                        {
                            jaConvertedDiagnoses.Add(jaDiagnoses[i]);
                        }
                    }
                    else
                    {
                        if (jaDiagnoses[i]["diaTypeCode"].ToString() == "ADD") //ADD 入院 OPD 门诊 DIS 出院
                        {
                            jaConvertedDiagnoses.Add(jaDiagnoses[i]);
                        }
                    }
                }

                for (int i = 0; i < jaConvertedDiagnoses.Count; i++)
                {
                    jaConvertedDiagnoses[i]["diag_srt_no"] = i + 1;
                    if (jaConvertedDiagnoses[i]["maindiag_flag"].ToString() == "1")
                    {
                        dscg_maindiag_code = jaConvertedDiagnoses[i]["diag_code"].ToString();
                        dscg_maindiag_name = jaConvertedDiagnoses[i]["diag_name"].ToString();
                    }
                    //增加就诊ID
                    jaConvertedDiagnoses[i]["mdtrt_id"] = new JObject();
                    jaConvertedDiagnoses[i]["mdtrt_id"] = Global.pat.mdtrtID;
                }

                frmReg.DtDiagnose = (DataTable)jaConvertedDiagnoses.ToObject(typeof(DataTable));
                if (frmReg.ShowDialog() == DialogResult.OK)
                {                    
                    string diseCodg = "";
                    string diseName = "";

                    //出院登记入参
                    //单病种出院登记 病种信息取入院登记入参
                    if (Global.pat.RYorCY == "2")
                    {
                        if (Global.pat.medType != "2102")
                        {
                            frmReg.insuType = Global.pat.insuType;
                        }

                        diseCodg = Global.pat.DiseasecCode; //取医保平台登记表的病种
                        diseName = Global.pat.DiseasecName;
                    }

                    //兰州要求必须传病种。普通疾病传主诊断
                    if ((frmReg.med_type == "2101") && (diseCodg == "") && (Global.pat.RYorCY == "2"))
                    {
                        if (string.IsNullOrEmpty(frmReg.diseCodg))
                        {
                            diseCodg = JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_code");
                        };
                        if (string.IsNullOrEmpty(frmReg.diseName))
                        {
                            diseName = JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_name");
                        };
                    }

                    JObject joDscgInfo = new JObject();
                    joDscgInfo.Add("mdtrt_id", Global.pat.mdtrtID);
                    joDscgInfo.Add("psn_no", Global.pat.psn_no);          //人员编号
                    joDscgInfo.Add("insutype", frmReg.insuType);          //险种类型
                    //joDscgInfo.Add("coner_name", JsonHelper.getJsonValue(joHisPatInfo, "coner_name"));          //联系人姓名
                    //joDscgInfo.Add("tel", JsonHelper.getJsonValue(joHisPatInfo, "tel"));          //联系电话
                    joDscgInfo.Add("endtime", Global.Set.hisDischargeTime);          //结束时间DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")
                    joDscgInfo.Add("dise_codg", diseCodg);          //病种编码
                    joDscgInfo.Add("dise_name", diseName);          //病种名称
                    joDscgInfo.Add("oprn_oprt_code", frmReg.oprn_oprt_code);          //手术操作代码
                    joDscgInfo.Add("oprn_oprt_name", frmReg.oprn_oprt_name);          //手术操作名称
                    joDscgInfo.Add("fpsc_no", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.fpsc_no"));          //计划生育服务证号
                    joDscgInfo.Add("matn_type", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.matn_type"));          //生育类别
                    joDscgInfo.Add("birctrl_type", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.birctrl_type"));          //计划生育手术类别
                    joDscgInfo.Add("latechb_flag", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.latechb_flag"));          //晚育标志
                    joDscgInfo.Add("geso_val", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.geso_val"));          //孕周数
                    joDscgInfo.Add("fetts", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.fetts"));          //胎次
                    joDscgInfo.Add("fetus_cnt", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.fetus_cnt"));          //胎儿数
                    joDscgInfo.Add("pret_flag", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.pret_flag"));          //早产标志
                    joDscgInfo.Add("birctrl_matn_date", "");          //计划生育手术或生育日期 JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.birctrl_matn_date") 20230228医保中心校验传正确时间或空
                    joDscgInfo.Add("cop_flag", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dise_type_code"));          //伴有并发症标志
                    joDscgInfo.Add("dscg_dept_codg", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_dept_codg"));          //出院科室编码
                    joDscgInfo.Add("dscg_dept_name", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_dept_name"));          //出院科室名称
                    joDscgInfo.Add("dscg_bed", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_bed"));          //出院床位
                    joDscgInfo.Add("dscg_way", "1");          //离院方式
                    joDscgInfo.Add("die_date", "");          //死亡日期
                    joDscgInfo.Add("exp_content", "");          //字段扩展
                    joDscgInfo.Add("insuplc_admdvs", Global.pat.insuplc_admdvs);          //参保地医保区划
                    joDscgInfo.Add("mdtrtarea_admvs", Global.inf.areaCode);          //就医地医保区划

                    JObject joRegInpar = new JObject();
                    joRegInpar.Add("dscginfo", joDscgInfo);
                    joRegInpar.Add("diseinfo", jaConvertedDiagnoses);


                    //结算入参
                    JObject joSetlInpar = new JObject();
                    joSetlInpar.Add("psn_no", Global.pat.psn_no);
                    joSetlInpar.Add("chrg_bchno", Global.pat.adm_Dr.ToString());
                    joSetlInpar.Add("mdtrt_cert_type", Global.pat.mdtrtcertType);
                    joSetlInpar.Add("mdtrt_cert_no", Global.pat.mdtrtcertNO);
                    joSetlInpar.Add("med_type", Global.pat.medType);
                    joSetlInpar.Add("medfee_sumamt", JsonHelper.getDestValue(joSumFee, "data.Sumamt"));
                    joSetlInpar.Add("psn_setlway", frmReg.psn_setlway); //结算方式
                    joSetlInpar.Add("mdtrt_id", Global.pat.mdtrtID);
                    joSetlInpar.Add("acct_used_flag", frmReg.personAccountUsedFlag);//个人账户使用标志
                    joSetlInpar.Add("insutype", frmReg.insuType);//险种类型
                    //joSetlInpar.Add("insuplc_admdvs", Global.pat.insuplc_admdvs);
                    //joSetlInpar.Add("mdtrtarea_admvs", Global.inf.areaCode);
                    joSetlInpar.Add("invono", "");
                    joSetlInpar.Add("mid_setl_flag", "0");
                    joSetlInpar.Add("fulamt_ownpay_amt", JsonHelper.getDestValue(joSumFee, "data.FullOwnPayAmount"));
                    joSetlInpar.Add("overlmt_selfpay", JsonHelper.getDestValue(joSumFee, "data.OverLimitAmount"));
                    joSetlInpar.Add("preselfpay_amt", JsonHelper.getDestValue(joSumFee, "data.PreSelfPayAmount"));
                    joSetlInpar.Add("dscgTime", Global.Set.hisDischargeTime);

                    JObject joDataExp = new JObject();
                    joDataExp.Add("acct_payamt", "0.00");
                    joDataExp.Add("cla_trt_flag", "");

                    joSetlInpar.Add("exp_content", joDataExp); //11111

                    JObject joOut = new JObject();
                    joOut.Add("DischargeRegister", joRegInpar);
                    joOut.Add("Settlement", joSetlInpar);
                    joOut.Add("errorCode", 0);
                    outParam = joOut.ToString();
                    return 0;

                }
                else
                {
                    outParam = "收款员取消结算";
                    return -1;
                }
            }
            catch (Exception ex)
            {
                outParam = "showDischargeRegisterForm 异常：" + ex.Message;
                return -1;
            }
        }
        public int showOutPatRegisterForm(out string regInfo)
        {
            
            //调用服务获取门诊诊断信息
            string diagnoses;
            if (hIS.getPatDiagnoses(Global.pat, out diagnoses) != 0)
            {
                regInfo = JsonHelper.setExceptionJson(-1, "getPatDiagnoses", diagnoses).ToString();
                return -1;
            }
            JObject joHisRtnInfo = JObject.Parse(diagnoses);            

            //组织入参
            JObject joReg = new JObject();
            try
            {
                JArray jaDiagnoses = JArray.Parse(JsonHelper.getDestValue(joHisRtnInfo, "diseinfo"));
                //登记面板实例
                OutpatientRegistration frmReg = new OutpatientRegistration();
                //frmReg.dblkcbxSettelmentWay.Enabled = false;

                //险种类型
                DataTable dtInsuType = (DataTable)frmReg.dblkcbxInsuranceType.DataSource;
                for (int i = 0; i < dtInsuType.Rows.Count; i++)
                {
                    if (dtInsuType.Rows[i]["Code"].ToString() == Global.pat.insuType)
                    {
                        frmReg.dblkcbxInsuranceType.Text = dtInsuType.Rows[i]["Name"].ToString();
                        frmReg.insuType = dtInsuType.Rows[i]["Code"].ToString();
                    }
                }

                //医疗类别
                DataTable dtMedType = (DataTable)frmReg.dblkcbxMedicalType.DataSource;
                for (int i = 0; i < dtMedType.Rows.Count; i++)
                {
                    if (dtMedType.Rows[i]["Code"].ToString() == "11")
                    {
                        frmReg.dblkcbxMedicalType.Text = dtMedType.Rows[i]["Name"].ToString();
                        frmReg.med_type = dtMedType.Rows[i]["Code"].ToString();
                    }
                }

                //结算方式
                DataTable dtSettlWay = (DataTable)frmReg.dblkcbxSettelmentWay.DataSource;
                for (int i = 0; i < dtSettlWay.Rows.Count; i++)
                {
                    if (dtSettlWay.Rows[i]["Code"].ToString() == "01")
                    {
                        frmReg.dblkcbxSettelmentWay.Text = dtSettlWay.Rows[i]["Name"].ToString();
                        frmReg.psn_setlway = dtSettlWay.Rows[i]["Code"].ToString();
                    }
                }

                //转换下主要诊断
                for (int i = 0; i < jaDiagnoses.Count; i++)
                {
                    string diag_type = JsonHelper.getDestValue((JObject)jaDiagnoses[i], "diag_type");
                    switch (diag_type)
                    {
                        case "1":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "西医主要诊断";
                                break;
                            }
                        case "2":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "西医其他诊断";
                                break;
                            }
                        case "3":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "中医主病诊断";
                                break;
                            }
                        case "4":
                            {
                                jaDiagnoses[i]["diag_type_name"] = new JObject();
                                jaDiagnoses[i]["diag_type_name"] = "中医主证诊断";
                                break;
                            }
                    }

                }
                frmReg.DtDiagnose = (DataTable)jaDiagnoses.ToObject(typeof(DataTable));
                //展示登记面板
                if (frmReg.ShowDialog() == DialogResult.OK)
                {
                    string diseCodg = "", diseName = "";
                    //哈尔滨要求必须传病种。普通疾病传主诊断
                    if (string.IsNullOrEmpty(frmReg.diseCodg))
                    {
                        diseCodg = JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_code");
                    };
                    if (string.IsNullOrEmpty(frmReg.diseName))
                    {
                        diseName = JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.dscg_maindiag_name");
                    };
                    //组织挂号入参
                    JObject joRegInpar = new JObject();
                    string occurTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    joRegInpar.Add("psn_no", Global.pat.psn_no);
                    joRegInpar.Add("insutype", frmReg.insuType);
                    joRegInpar.Add("begntime", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.begntime"));//挂号时间  这里之所以不取当前时间是因为就诊信息上传会关联判断挂号的时间DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")
                    joRegInpar.Add("mdtrt_cert_type", Global.pat.mdtrtcertType);
                    joRegInpar.Add("mdtrt_cert_no", Global.pat.mdtrtcertNO);
                    joRegInpar.Add("ipt_otp_no", Global.pat.adm_Dr);
                    joRegInpar.Add("atddr_no", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.atddr_no"));
                    joRegInpar.Add("dr_name", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.chfpdr_name"));
                    joRegInpar.Add("dept_code", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_dept_codg"));
                    joRegInpar.Add("dept_name", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.adm_dept_name"));
                    joRegInpar.Add("exp_content", "");
                    joRegInpar.Add("caty", "72");
                    joReg.Add("data", joRegInpar);

                    //组织就诊信息入参
                    JObject joMdtrtinfo = new JObject();
                    for (int i = 0; i < jaDiagnoses.Count; i++)
                    {
                        jaDiagnoses[i]["vali_flag"] = new JObject();
                        jaDiagnoses[i]["vali_flag"] = "1";
                    }
                    joMdtrtinfo.Add("mdtrt_id", Global.pat.mdtrtID);
                    joMdtrtinfo.Add("psn_no", Global.pat.psn_no);          //人员编号
                    Global.pat.medType = frmReg.med_type;
                    joMdtrtinfo.Add("med_type", frmReg.med_type);          //医疗类别
                    joMdtrtinfo.Add("begntime", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.begntime"));          //开始时间
                    joMdtrtinfo.Add("main_cond_dscr", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.main_cond_dscr"));          //主要病情描述
                    joMdtrtinfo.Add("dise_codg", diseCodg);          //病种编码
                    joMdtrtinfo.Add("dise_name", diseName);          //病种名称
                    joMdtrtinfo.Add("birctrl_type", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.birctrl_type"));          //计划生育手术类别
                    joMdtrtinfo.Add("birctrl_matn_date", "");          //计划生育手术或生育日期 JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.birctrl_matn_date") 20230228医保中心校验传正确时间或空
                    joMdtrtinfo.Add("matn_type", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.matn_type"));          //生育类别
                    joMdtrtinfo.Add("geso_val", JsonHelper.getDestValue(joHisRtnInfo, "mdtrtinfo.geso_val"));            //孕周数
                    joMdtrtinfo.Add("exp_content", "");          //字段扩展

                    //组织结算入参
                    JObject joSetlInpar = new JObject();
                    joSetlInpar.Add("psn_no", Global.pat.psn_no);
                    joSetlInpar.Add("mdtrt_cert_type", Global.pat.mdtrtcertType);
                    joSetlInpar.Add("mdtrt_cert_no", Global.pat.mdtrtcertNO);
                    joSetlInpar.Add("med_type", Global.pat.medType);
                    joSetlInpar.Add("medfee_sumamt", "");
                    joSetlInpar.Add("psn_setlway", frmReg.psn_setlway); //结算方式
                    joSetlInpar.Add("mdtrt_id", Global.pat.mdtrtID);
                    joSetlInpar.Add("chrg_bchno", Global.pat.adm_Dr.ToString());
                    joSetlInpar.Add("acct_used_flag", frmReg.personAccountUsedFlag);//个人账户使用标志
                    joSetlInpar.Add("insutype", frmReg.insuType);//险种类型
                    joSetlInpar.Add("pub_hosp_rfom_flag", "");
                    joSetlInpar.Add("exp_content11", "");


                    joReg.Add("mdtrtinfo", joMdtrtinfo);
                    joReg.Add("diseinfo", jaDiagnoses);
                    joReg.Add("settlement", joSetlInpar);

                    regInfo = joReg.ToString();
                    return 0;
                }
                else
                {
                    regInfo = "收费员取消操作！";
                    return -1;
                }
            }
            catch (Exception ex)
            {
                regInfo = "登记界面显示异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 上传费用到中心
        /// </summary>
        /// <returns></returns>
        /// 
        public int  uploadFee(string funNO,int singleCount,out string outParam)
        {
            outParam = "";
            //获取IS费用
            if (hIS.getHisFee(Global.pat, out outParam) != 0)
            {
                return -1;
            }
            //调用医保平台转换HIS费用（转换医保编码等）
            JObject joHisFee = JObject.Parse(outParam);

            if (mIS.convertHisFeeWithInsuCode(joHisFee, out outParam) != 0)
            {
                return -2;
            }

            JArray jaFeeDetail = JArray.Parse(JsonHelper.getDestValue(JObject.Parse(outParam), "data"));
            //按指定条数分割后上传，保存，更新
            if (uploadFeeToCenter(funNO, singleCount, jaFeeDetail, out outParam) != 0)
            {
                return -3;
            }
            else
            {
                outParam = "成功";
                return 0;
            }
            
        }

        public int uploadFeeToCenter(string funNO,int singleCount,JArray jaFeeDetail, out string outParam)
        {
            outParam = "";
            string errMsg;
            try
            {
                JArray jaTmp = new JArray();
                for (int i = 0; i < jaFeeDetail.Count; i++)
                {
                    jaFeeDetail[i]["chrg_bchno"] = Global.pat.adm_Dr.ToString();
                    jaFeeDetail[i]["med_type"] = Global.pat.medType;
                    if (jaFeeDetail[i]["validFlag"].ToString() == "N") continue;
                    jaTmp.Add(jaFeeDetail[i]);
                    if (((i + 1) % singleCount == 0) || ((i + 1) == jaFeeDetail.Count))
                    {
                        JObject joFeeDetail = new JObject();
                        joFeeDetail.Add("feedetail", jaTmp);
                        JObject joRtn = invoker.invokeCenterService(funNO, JsonHelper.setCenterInpar(funNO, joFeeDetail.ToString()));
                        if (JsonHelper.parseCenterRtnValue(joRtn, out errMsg) != 0)
                        {
                            outParam = errMsg;
                            return -1;
                        }
                        else
                        {                            
                            //成功后插入医保平台
                            if (mIS.insertFee(jaTmp, out errMsg) != 0)
                            {
                                outParam = outParam + errMsg;
                                return -1;
                            }
                            //更新2301返回的数据
                            if (mIS.updateFee(JArray.Parse(JsonHelper.getDestValue(joRtn, "output.result")), out errMsg) != 0)
                            {
                                outParam = outParam + errMsg;
                                return -1;
                            }                            
                        }

                        jaTmp = new JArray();
              
                    }

                }
                return 0;
            }
            catch (Exception ex)
            {
                outParam = "uploadFee 异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 获取登记时候传入的信息，并赋值给patients结构体
        /// </summary>
        /// <param name="businessType"></param>
        /// <param name="inparam"></param>
        /// <param name="pat"></param>
        public void GetRegPatInfo(string businessType, JObject joInparam,out Patients pat)
        {
            pat = new Patients();
            try
            {
                
                switch (businessType)
                {
                    case "M2"://门诊登记
                        {
                            break;
                        }
                    case "M2C"://门诊登记撤销
                        {
                            break;
                        }
                    case "Z0"://住院修改
                        {
                            break;
                        }
                    case "Z2"://入院登记
                        {
                            pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                            pat.psn_no = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.output.baseinfo.psn_no");
                            pat.insuType = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.selectedInsuInfo.insutype");
                            pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.selectedInsuInfo.insuplc_admdvs");
                            pat.mdtrtcertType = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.selectedInsuInfo.mdtrt_cert_type");
                            pat.mdtrtcertNO = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.selectedInsuInfo.mdtrt_cert_no");
                            pat.name = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.output.baseinfo.psn_name");
                            pat.psn_type = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.selectedInsuInfo.psn_type");
                            pat.certType = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.output.baseinfo.psn_cert_type");
                            pat.certNO = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.output.baseinfo.certno");
                            pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                            pat.card.SN = JsonHelper.getDestValue(joInparam, "params[0].patientInfo.cardinfo.card_sn");
                            break;
                        }
                    case "Z2C"://入院登记取消
                        {
                            pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                            pat.psn_no = JsonHelper.getDestValue(joInparam, "params[0].psn_no");
                            pat.mdtrtID = JsonHelper.getDestValue(joInparam, "params[0].mdtrt_id");
                            //pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "params[0].insuplc_admdvs");
                            pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                            break;
                        }
                    case "Z6"://出院登记
                        {
                            Global.Set.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")).ToString("yyyy-MM-dd HH:mm:ss");

                            break;
                        }
                    case "Z6C"://出院登记取消
                        {
                            pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                            pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                            pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                            break;
                        }
                    default:
                        {
                            break;
                        }
                }
                //如果患者异地的则就医地需要该为419900
                if (Global.pat.insuplc_admdvs != "")
                {
                    if (Global.pat.insuplc_admdvs.Substring(0, 4) != Global.inf.areaCode.Substring(0, 4))
                    {
                        Global.inf.areaCode = "419900";
                    }
                }
            }
            catch (Exception ex)
            {
                Global.writeLog("GetRegPatInfo(" + businessType + ")", JsonHelper.Compress(joInparam),"PAT结构体设置异常：" + ex.Message);
            }
        }

        /// <summary>
        /// 获取费用上传时前端传入的信息，并赋值给patients结构体
        /// </summary>
        /// <param name="businessType"></param>
        /// <param name="joInparam"></param>
        /// <param name="pat"></param>
        public void GetFeeHisInfo(string businessType, JObject joInparam, out Patients pat)
        {
            pat = new Patients();
            try
            {
                switch (businessType)
                {
                    case "M3"://门诊费用上传
                        {
                            break;
                        }
                    case "M3C"://门诊费用撤销
                        {

                            break;
                        }
                    case "Z3"://入院费用上传
                        {
                            pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.admID"));
                            pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                            pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                            pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                            pat.name = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_name");
                            pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                            pat.recordID = "";
                            break;
                        }
                    case "Z3C"://住院费用上传取消
                        {
                            pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.admID"));
                            pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                            pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                            pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                            break;
                        }

                    default:
                        {
                            break;
                        }

                }
                //如果患者异地的则就医地需要该为369900
                if (Global.pat.insuplc_admdvs != "")
                {
                    if (Global.pat.insuplc_admdvs.Substring(0, 4) != Global.inf.areaCode.Substring(0, 4))
                    {
                        Global.inf.areaCode = "369900";
                    }
                }
            }
            catch (Exception ex)
            {
                Global.writeLog("GetFeeHisInfo(" + businessType + ")", JsonHelper.Compress(joInparam), "PAT结构体设置异常：" + ex.Message);
            }


        }

        /// <summary>
        ///  获取结算时前端传入的信息，并赋值给patients结构体
        /// </summary>
        /// <param name="businessType"></param>
        /// <param name="joInparam"></param>
        /// <param name="pat"></param>
        public void GetSettlementHisInfo(string businessType, JObject joInparam, out Patients pat)
        {
            pat = new Patients();
            switch (businessType)
            {
                case "M4"://门诊预结算
                    {

                        break;
                    }
                case "M4C"://门诊预结算撤销
                    {


                        break;
                    }
                case "M5"://门诊结算
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        pat.recordID = JsonHelper.getDestValue(joInparam, "params[0].recordID");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        Global.pat.insuplc_admdvs = "";
                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays")))
                        {
                            Global.Set.hospitalizationsDays = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsDays = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes")))
                        {
                            Global.Set.hospitalizationsTimes = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsTimes = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")))
                        {
                            Global.Set.hisAdmTime = "";
                        }
                        else
                        {
                            Global.Set.hisAdmTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")))
                        {
                            Global.Set.hisDischargeTime = "";
                        }
                        else
                        {
                            Global.Set.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }
                        //Global.settl.hospitalizationsDays = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays"));
                        //Global.settl.hospitalizationsTimes = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes"));
                        //Global.settl.hisAdmTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime"));
                        //Global.settl.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime"));
                        Global.writeLog(Global.Set.hospitalizationsDays.ToString());
                        Global.writeLog(Global.Set.hospitalizationsTimes.ToString());
                        Global.writeLog(Global.Set.hisAdmTime.ToString());
                        Global.writeLog(Global.Set.hisDischargeTime.ToString());

                        break;
                    }
                case "M5C"://门诊结算撤销
                    {
                        pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                        pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                        pat.settlID = JsonHelper.getDestValue(joInparam, "insuAdmObj.setl_id");
                        pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        pat.recordID = JsonHelper.getDestValue(joInparam, "params[0].recordID");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        break;
                    }
                case "Z4"://住院预结算
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.admID"));
                        pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                        pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                        pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                        pat.name = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_name");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays")))
                        {
                            Global.Set.hospitalizationsDays = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsDays = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes")))
                        {
                            Global.Set.hospitalizationsTimes = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsTimes = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")))
                        {
                            Global.Set.hisAdmTime = "";
                        }
                        else
                        {
                            Global.Set.hisAdmTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")))
                        {
                            Global.Set.hisDischargeTime = "";
                        }
                        else
                        {
                            Global.Set.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }
                        //Global.settl.hospitalizationsDays = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays"));
                        //Global.settl.hospitalizationsTimes = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes"));
                        //Global.settl.hisAdmTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime"));
                        //Global.settl.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime"));
                        Global.writeLog(Global.Set.hospitalizationsDays.ToString());
                        Global.writeLog(Global.Set.hospitalizationsTimes.ToString());
                        Global.writeLog(Global.Set.hisAdmTime.ToString());
                        Global.writeLog(Global.Set.hisDischargeTime.ToString());
                        break;
                    }
                case "Z4C"://住院预结算撤销
                    {
                        pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                        pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                        pat.settlID = JsonHelper.getDestValue(joInparam, "insuAdmObj.setl_id");                        
                        pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        pat.recordID = JsonHelper.getDestValue(joInparam, "params[0].recordID");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        break;
                    }
                case "Z5"://住院结算
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.admID"));
                        pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                        pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                        pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                        pat.name = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_name");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        //pat.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays")))
                        {
                            Global.Set.hospitalizationsDays = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsDays = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes")))
                        {
                            Global.Set.hospitalizationsTimes = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsTimes = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")))
                        {
                            Global.Set.hisAdmTime = "";
                        }
                        else
                        {
                            Global.Set.hisAdmTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")))
                        {
                            Global.Set.hisDischargeTime = "";
                        }
                        else
                        {
                            Global.Set.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }
                        //Global.settl.hospitalizationsDays = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays"));
                        //Global.settl.hospitalizationsTimes = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes"));
                        //Global.settl.hisAdmTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime"));
                        //Global.settl.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime"));

                        break;
                    }
                case "Z5C"://住院结算撤销
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.admID"));
                        pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                        pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                        pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                        pat.name = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_name");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        pat.settlID = JsonHelper.getDestValue(joInparam, "insuAdmObj.setl_id");
                        pat.SettID_HISJSB = pat.settlID;  //这里是通过09010059交易获取到的结算ID
                        break;
                    }
                default:
                    {
                        break;
                    }
            }
            //如果患者异地的则就医地需要该为369900
            //if (Global.pat.insuplc_admdvs != "")
            //{
            //    if (Global.pat.insuplc_admdvs.Substring(0, 4) != Global.inf.areaCode.Substring(0, 4))
            //    {
            //        Global.inf.areaCode = "369900";
            //    }
            //}
            
        }


        /// <summary>
        /// 获取自助机调用时前端传入的信息，并赋值给patients结构体
        /// </summary>
        /// <param name="businessType"></param>
        /// <param name="joInparam"></param>
        /// <param name="pat"></param>
        public void GetSelfServiceMachineInfo(string businessType, JObject joInparam, out Patients pat)
        {
            pat = new Patients();
            switch (businessType)
            {
                case "GetPatientInfo"://读取患者信息
                    {
                        Global.ssmp.cardType = JsonHelper.getDestValue(joInparam, "params[0].cardType");
                        Global.operationType = JsonHelper.getDestValue(joInparam, "operationType") + "-" + businessType;
                        break;
                    }
                case "OPReg"://门诊挂号
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        pat.recordID = JsonHelper.getDestValue(joInparam, "params[0].recordID");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        Global.ssmp.cardType = JsonHelper.getDestValue(joInparam, "params[0].cardType");
                        Global.pat.insuplc_admdvs = "";
                        Global.operationType = JsonHelper.getDestValue(joInparam, "operationType");
                        break;
                    }
                case "OPSettl"://门诊结算
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        pat.recordID = JsonHelper.getDestValue(joInparam, "params[0].recordID");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        pat.insuplc_admdvs = "";
                        Global.ssmp.cardType = JsonHelper.getDestValue(joInparam, "params[0].cardType");
                        Global.operationType = JsonHelper.getDestValue(joInparam, "operationType");
                        break;
                    }
                case "IPInReg"://入院登记
                    {
                        Global.ssmp.cardType = JsonHelper.getDestValue(joInparam, "params[0].cardType");
                        Global.operationType = JsonHelper.getDestValue(joInparam, "operationType") + "-" + businessType;
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        break;
                    }
                case "IPSettl"://住院结算
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        pat.psn_no = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_no");
                        pat.mdtrtID = JsonHelper.getDestValue(joInparam, "insuAdmObj.mdtrt_id");
                        pat.name = JsonHelper.getDestValue(joInparam, "insuAdmObj.psn_name");
                        pat.recordID = JsonHelper.getDestValue(joInparam, "params[0].recordID");
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        pat.recordID = JsonHelper.getDestValue(joInparam, "params[0].recordID");
                        pat.insuplc_admdvs = JsonHelper.getDestValue(joInparam, "insuAdmObj.insuplc_admdvs");
                        Global.ssmp.cardType = JsonHelper.getDestValue(joInparam, "params[0].cardType");

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays")))
                        {
                            Global.Set.hospitalizationsDays = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsDays = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inDays"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes")))
                        {
                            Global.Set.hospitalizationsTimes = 0;
                        }
                        else
                        {
                            Global.Set.hospitalizationsTimes = int.Parse(JsonHelper.getDestValue(joInparam, "insuAdmObj.inTimes"));
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")))
                        {
                            Global.Set.hisAdmTime = "";
                        }
                        else
                        {
                            Global.Set.hisAdmTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.admDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }

                        if (string.IsNullOrEmpty(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")))
                        {
                            Global.Set.hisDischargeTime = "";
                        }
                        else
                        {
                            Global.Set.hisDischargeTime = Convert.ToDateTime(JsonHelper.getDestValue(joInparam, "insuAdmObj.disDateTime")).ToString("yyyy-MM-dd HH:mm:ss");
                        }
                        break;
                    }
                case "DischargeAudit"://出院核查
                    {
                        pat.adm_Dr = int.Parse(JsonHelper.getDestValue(joInparam, "params[0].admID"));
                        pat.billID = JsonHelper.getDestValue(joInparam, "params[0].billID");
                        break;
                    }
                    

                default:
                    {
                        break;
                    }
            }

        }

        ///对目录下载业务的封装
        ///
        public JObject DownloadDirectory(string funNO,string inParam)
        {
            string errorMessage = "", sRtn = "";
            JObject joRtn = new JObject();
            try
            {

                JObject joCenterRtn = cBus.DownDirecotry(funNO,inParam);
                if (JsonHelper.parseCenterRtnValue(joCenterRtn,out errorMessage) != 0)
                {
                    joRtn = JsonHelper.setExceptionJson(-100, "下载异常", errorMessage);
                    return joRtn;
                }
                string filename = JsonHelper.getDestValue(joCenterRtn, "output.filename");

                //MessageBox.Show("1306查询目录文件号返回："+ joCenterRtn.ToString());

                JObject jo9102Rtn = cBus.SaveFile(joCenterRtn);
                if (JsonHelper.parseIrisRtnValue(jo9102Rtn,out errorMessage) != 0)
                {
                    joRtn = JsonHelper.setExceptionJson(-100, "保存文件异常", errorMessage);
                    return joRtn;
                }
                //解压缩
                string zipFilePath = jo9102Rtn["filePath"].ToString();
                string txtFileDir = System.Environment.CurrentDirectory + "\\Download";
                string txtFilePath = txtFileDir + "\\" + filename.Substring(0, filename.Length - 4);

                //string txtFilePath = Global.curEvt.path + "\\Download\\" + filename;

                //MessageBox.Show("解压缩的txtFilePath："+ zipFilePath+"|解压到目标路径："+ txtFileDir+"|文件解析路径："+ txtFilePath);

                //昆明下载下来的文件是解压后的TXT
                if (tools.ExtractFile(zipFilePath, txtFileDir, out errorMessage) != 0)
                {
                    joRtn = JsonHelper.setExceptionJson(-100, "提取文件", errorMessage);
                    return joRtn;
                }

                joRtn = (JObject)jo9102Rtn.DeepClone();
                joRtn["filePath"] = txtFilePath;
                return joRtn;
            }
            catch (Exception ex)
            {
                joRtn = JsonHelper.setExceptionJson(-100, "下载中心目录", errorMessage);
                return joRtn;
            }
            finally
            {
                Global.writeLog("HisMainBusiness.DownloadDirectory", inParam, joRtn.ToString());
            }
        }

        /// <summary>
        /// 字典下载
        /// </summary>
        /// <param name="inParam"></param>
        /// <returns></returns>
        public string downDictionary(string inParam)
        {
            JObject joCenterRtn = invoker.invokeCenterService("1901", inParam);
            return joCenterRtn.ToString();
        }       

        /// <summary>
        /// 展示结算信息界面
        /// </summary>
        /// <param name="joSettlement"></param>
        /// <param name="outParam"></param>
        /// <returns></returns>
        public int showSettlementForm(JObject joSettlement, out string outParam)
        {
            outParam = "";
            string errMsg;
            try
            {
                SettlementInfo frmSettlment = new SettlementInfo(joSettlement);
                //相关信息转换到结算信息面板
                if (frmSettlment.ShowDialog() == DialogResult.OK)
                {
                    return 0;
                }
                else
                {
                    outParam = "收款员取消";
                    return -1;
                }
            }
            catch (Exception ex)
            {
                outParam = "showSettlementForm 异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 门诊预结算封装，返回正式结算的入参
        /// </summary>
        /// <param name="joSettlement"></param>
        /// <param name="outParam"></param>
        /// <returns></returns>
        public int preSettlement_OutPat( JObject joSettlement, out string outParam)
        {
            outParam = "";

            //预结算  
            JObject jo2206Data = new JObject();
            jo2206Data.Add("data", joSettlement);
            JObject jo2206Rtn = invoker.invokeCenterService("2206", JsonHelper.setCenterInpar("2206", jo2206Data));
            if (JsonHelper.parseCenterRtnValue(jo2206Rtn, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "出院预结算", outParam).ToString();
                return -1;
            }

            //返回给HIS进行预结算判断
            JObject joSetlinfo = JObject.Parse(JsonHelper.getDestValue(jo2206Rtn, "output.setlinfo"));
            if (hIS.preSettlement(joSettlement, joSetlinfo, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "返回结算信息给HIS", outParam).ToString();
                return -1;
            }


            //编码转换
            if (mIS.convertSettlementWithInsuCode(jo2206Rtn, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "结算信息转换", outParam).ToString();
                return -1;
            }

            JObject joSumFee = new JObject();
            string errMsg = "";
            //判断勾稽关系是否平
            if (hIS.sumInsuRtnSettlInfo(JObject.Parse(JsonHelper.getDestValue(jo2206Rtn, "output.setlinfo")), out joSumFee, out errMsg) != 0)
            {
                outParam = "勾稽关系不符合标准，请联系管理员！" + errMsg;
                return -1;
            }



            //展示结算信息
            JObject joConverted = JObject.Parse(outParam);
            if (showSettlementForm(joConverted, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "结算信息展示", outParam).ToString();
                return -1;
            }
            else
            {
                return 0;
            }
        }

        /// <summary>
        /// 预结算封装，返回正式结算的入参
        /// </summary>
        /// <param name="joSettlement"></param>
        /// <param name="outParam"></param>
        /// <returns></returns>
        public int preSettlement(string funNO,out JObject joSettlement, out string outParam)
        {
            joSettlement = new JObject();
            outParam = "";
            string outPar = "";
            //调用IRIS获取医保各项金额
            if (mIS.getSumFee(out outPar) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "获取医保费用各项汇总金额", outPar).ToString();
                return -1;
            }
            JObject joSumFee = JObject.Parse(outPar);

            if ((JsonHelper.getDestValue(joSumFee, "data.Sumamt") == "") || (JsonHelper.getDestValue(joSumFee, "data.Sumamt") == "0"))
            {
                outParam = "请先右击选择【医保费用上传】！";
                return -1;
            }
            //从云医保平台获取患者详细的医保登记信息
            int type;
            if (funNO == "2206")
            {
                type = 3;  //门诊预结算
            }
            else
            {
                type = 1;  //住院预结算
            }
            if (mIS.queryRegisterInfo(type, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "查询患者云平台登记信息", outParam).ToString();
                return -1;
            }

            JObject joReg = JObject.Parse(outParam);
            Global.pat.medType = JsonHelper.getDestValue(joReg, "data.MedicalType");
            Global.pat.insuType = JsonHelper.getDestValue(joReg, "data.InsuType");

            if ((Global.pat.mdtrtcertType == "") || (Global.pat.mdtrtcertNO == ""))
            {
                Global.pat.mdtrtcertType = JsonHelper.getDestValue(joReg, "data.CertificateType");
                Global.pat.mdtrtcertNO = JsonHelper.getDestValue(joReg, "data.CertificateNO");
            }

            //Global.pat.mdtrtcertType = JsonHelper.getDestValue(joReg, "data.CertificateType");
            //Global.pat.mdtrtcertNO = JsonHelper.getDestValue(joReg, "data.CertificateNO");

            Global.pat.certType = JsonHelper.getDestValue(joReg, "data.CertificateType");
            Global.pat.certNO = JsonHelper.getDestValue(joReg, "data.CertificateNO");
            Global.pat.insuplc_admdvs = JsonHelper.getDestValue(joReg, "data.InsuranceAreaCode");
            Global.pat.psn_type = Global.pat.insuType;   //后面再改

            //if (Global.pat.RYorCY == "2")
            //{
            //    Global.pat.DiseasecCode = JsonHelper.getDestValue(joReg, "data.DiseasecCode");
            //    Global.pat.DiseasecName = JsonHelper.getDestValue(joReg, "data.DiseasecName");
            //}

            if ((Global.pat.medType == "2101") && (JsonHelper.getDestValue(joReg, "data.DiseasecCode") == "") && (Global.pat.RYorCY == "2"))            
            {
                Global.pat.DiseasecCode = JsonHelper.getDestValue(joReg, "data.MainDiagCode");
                Global.pat.DiseasecName = JsonHelper.getDestValue(joReg, "data.MainDiagName");
            }                        

            //调用服务获取门诊诊断信息
            string diagnoses;
            if (hIS.getPatDiagnoses(Global.pat, out diagnoses) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "getPatDiagnoses", diagnoses).ToString();
                return -1;
            }
            JObject joDiagnoses = JObject.Parse(diagnoses);
            //展示出院登记界面，选择结算方式等
            if (showDischargeRegisterForm(funNO, joSumFee, joDiagnoses, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "展示出院预结算界面", outParam).ToString();
                return -1;
            }           
            JObject joRtn = JObject.Parse(outParam);
            JObject jo2303Inpar = JObject.Parse(JsonHelper.getDestValue(joRtn, "Settlement"));
            JObject jo2402Inpar = JObject.Parse(JsonHelper.getDestValue(joRtn, "DischargeRegister"));

            //昆明出院登记放再预结算之前
            JObject jo2402Rtn = invoker.invokeCenterService("2402", JsonHelper.setCenterInpar("2402", jo2402Inpar));
            string errMsg;
            if (JsonHelper.parseCenterRtnValue(jo2402Rtn, out errMsg) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "预结算失败：出院登记失败", errMsg).ToString();
                return -1;
            }

            //预结算 
            JObject jo2303Data = new JObject();
            jo2303Data.Add("data", jo2303Inpar);
            JObject jo2303Rtn = invoker.invokeCenterService(funNO, JsonHelper.setCenterInpar(funNO, jo2303Data));
            if (JsonHelper.parseCenterRtnValue(jo2303Rtn, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "出院预结算", outParam).ToString();
                return -1;
            }

            //预结算结果传给HIS进行HIS判断 
            if (funNO == "2303")
            {
                //返回给HIS
                JObject joSetlinfo = JObject.Parse(JsonHelper.getDestValue(jo2303Rtn, "output.setlinfo"));
                JObject joParam = new JObject();
                joParam.Add("admID", Global.pat.adm_Dr);
                joParam.Add("recordID", Global.pat.recordID);
                joParam.Add("billID", Global.pat.billID);

                if (hIS.preSettlementInfo(joParam, joSetlinfo, out outParam) != 0)
                {
                    outParam = JsonHelper.setExceptionJson(-1, "出院预结算", outParam).ToString();
                    return -1;
                }
            }
            
            //编码转换
            if (mIS.convertSettlementWithInsuCode(jo2303Rtn, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "结算信息转换", outParam).ToString();
                return -1;
            }
            //展示结算信息
            JObject joConverted = JObject.Parse(outParam);
            if (showSettlementForm(joConverted, out outParam) != 0)
            {
                outParam = JsonHelper.setExceptionJson(-1, "结算信息展示", outParam).ToString();
                return -1;
            }
            else
            {
                joSettlement = (JObject)joRtn.DeepClone();
                return 0;
            }
        }

        public int returnToFrontEndAfterSettlement(JObject joCenterRtn,JObject joHisServieRtn, out string outParam)
        {
            outParam = "";
            try
            {
                JObject johisInsuInfo = JObject.Parse(JsonHelper.getDestValue(joHisServieRtn, "result"));
                johisInsuInfo.Add("setl_id", JsonHelper.getDestValue(joCenterRtn, "output.setlinfo.setl_id"));
                JObject joTmp = (JObject)joCenterRtn.DeepClone();              
                joTmp.Add("hisInsuInfo", johisInsuInfo);
                outParam = joTmp.ToString();
                return 0;
            }
            catch (Exception ex)
            {
                outParam = "returnToFrontEndAfterSettlement：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 插入患者该次就诊参保信息
        /// </summary>
        /// <param name="mdtrtID"></param>
        /// <param name="patInfo"></param>
        /// <param name="outparam"></param>
        /// <returns></returns>
        public int insertPatCurInsuInfo(String patInfo, out string outparam)
        {
            Global.writeLog(" " + patInfo);
            outparam = "";
            try
            {
                JObject joPatInfo = JObject.Parse(patInfo);
                JObject joBaseInfo = JObject.Parse(JsonHelper.getDestValue(joPatInfo, "output.baseinfo"));
                JObject joInsuInfo = JObject.Parse(JsonHelper.getDestValue(joPatInfo, "selectedInsuInfo"));
                JObject joIdetInfo = JObject.Parse(JsonHelper.getDestValue(joPatInfo, "selectedIdInfo"));
                if (mIS.insertPatCurInsuInfo(joBaseInfo, joInsuInfo, joIdetInfo, out outparam) != 0)
                {
                    return -1;
                }
                else
                {
                    return 0;
                }
                
            }
            catch (Exception e)
            {
                outparam =  "插入患者该次就诊参保信息失败：" + e.Message;
                return -1;
            }
        }



    }
}
